############################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
import ZSI
from ZSI.TC import _get_global_element_declaration as GED,\
    _get_type_definition as GTD
from ZSI.TC import ElementDeclaration
from ZSI.TC import TypeDefinition
from ZSI.generate.pyclass import pyclass_type
from ZSI.wstools.Namespaces import XMLNS

"""
Patches:
Generator problems and bugs in Specifications.
Just need to load this module FIRST in order for the patches to take affect.
"""

class BaseFaultType_Def(ZSI.TCcompound.ComplexType, TypeDefinition):
    """Patch for recursion BaseFaultType contains itself so must use
    lazy evaluation.  Should become obsolete.
    """
    schema = "http://docs.oasis-open.org/wsrf/2004/06/wsrf-WS-BaseFaults-1.2-draft-01.xsd"
    type = (schema, "BaseFaultType")
    def __init__(self, pname, ofwhat=(), attributes=None, extend=False, restrict=False, **kw):
        ns = BaseFaultType_Def.schema
        TClist = [ZSI.TCtimes.gDateTime(pname=(ns,"Timestamp"), aname="_Timestamp", minOccurs=1, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), 
                  GTD("http://schemas.xmlsoap.org/ws/2004/03/addressing","EndpointReferenceType",lazy=False)(pname=(ns,"Originator"), aname="_Originator", minOccurs=0, maxOccurs=1, nillable=False, typed=False, encoded=kw.get("encoded")), 
                  self.__class__.ErrorCode_Dec(minOccurs=0, maxOccurs=1, nillable=False, encoded=kw.get("encoded")), 
                  self.__class__.Description_Dec(minOccurs=0, maxOccurs="unbounded", nillable=False, encoded=kw.get("encoded")), 
                  GTD("http://docs.oasis-open.org/wsrf/2004/06/wsrf-WS-BaseFaults-1.2-draft-01.xsd","BaseFaultType",lazy=True)(pname=(ns,"FaultCause"), aname="_FaultCause", minOccurs=0, maxOccurs="unbounded", nillable=False, typed=False, encoded=kw.get("encoded")),
                  ]
        self.attribute_typecode_dict = attributes or {}
        if extend: TClist += ofwhat
        if restrict: TClist = ofwhat
        ZSI.TCcompound.ComplexType.__init__(self, None, TClist, pname=pname, inorder=0, **kw)
        class Holder:
            __metaclass__ = pyclass_type
            typecode = self
            def __init__(self):
                # pyclass
                self._Timestamp = None
                self._Originator = None
                self._ErrorCode = None
                self._Description = []
                self._FaultCause = []
                return
        Holder.__name__ = "BaseFaultType_Holder"
        self.pyclass = Holder
        
    class ErrorCode_Dec(ZSI.TCcompound.ComplexType):
        schema = "http://docs.oasis-open.org/wsrf/2004/06/wsrf-WS-BaseFaults-1.2-draft-01.xsd"
        literal = "ErrorCode"
        def __init__(self, **kw):
            ns = BaseFaultType_Def.ErrorCode_Dec.schema
            TClist = [ZSI.TC.AnyElement(aname="_any", minOccurs=0, maxOccurs="unbounded", nillable=False, processContents="lax")]
            kw["pname"] = ("http://docs.oasis-open.org/wsrf/2004/06/wsrf-WS-BaseFaults-1.2-draft-01.xsd","ErrorCode")
            kw["aname"] = "_ErrorCode"
            self.attribute_typecode_dict = {"dialect":ZSI.TC.URI()}
            kw.setdefault("mixed", True)
            kw.setdefault("mixed_aname", "_text")
            ZSI.TCcompound.ComplexType.__init__(self, None, TClist, inorder=0, **kw)
            class Holder:
                __metaclass__ = pyclass_type
                typecode = self
                def __init__(self):
                    # pyclass
                    return
            Holder.__name__ = "ErrorCode_Holder"
            self.pyclass = Holder

    class Description_Dec(ZSI.TC.String, ElementDeclaration):
        # Local ComplexType/SimpleContent derivation of built-in type
        schema = "http://docs.oasis-open.org/wsrf/2004/06/wsrf-WS-BaseFaults-1.2-draft-01.xsd"
        literal = "Description"
        def __init__(self, **kw):
            ns = BaseFaultType_Def.Description_Dec.schema
            pname = ("http://docs.oasis-open.org/wsrf/2004/06/wsrf-WS-BaseFaults-1.2-draft-01.xsd","Description")
            kw["aname"] = "_Description"
            self.attribute_typecode_dict = {(XMLNS.BASE,'lang'):ZSI.TC.String()}
            ZSI.TC.String.__init__(self, pname, **kw)
            class Holder(str):
                __metaclass__ = pyclass_type
                typecode = self
            Holder.__name__ = "Description_Holder"
            self.pyclass = Holder


class QueryResourcePropertiesResponse_Dec(ZSI.TCcompound.ComplexType, ElementDeclaration):
    """Patch for <any> minOccurs facet... In XML Schema it is required, however, 
    several examples provide for only a boolean result return in the mixed text content.
    Also there is a wsdl2py generator problem w/ComplexContent mixed=True, since
    this is not being generated correctly.
    """
    schema = "http://docs.oasis-open.org/wsrf/2004/06/wsrf-WS-ResourceProperties-1.2-draft-01.xsd"
    literal = "QueryResourcePropertiesResponse"
    def __init__(self, **kw):
        ns = QueryResourcePropertiesResponse_Dec.schema
        TClist = [\
            ZSI.TC.AnyElement(aname="_any", minOccurs=0, maxOccurs="unbounded", nillable=False, processContents="lax"),
        ]
        kw["pname"] = ("http://docs.oasis-open.org/wsrf/2004/06/wsrf-WS-ResourceProperties-1.2-draft-01.xsd","QueryResourcePropertiesResponse")
        kw["aname"] = "_QueryResourcePropertiesResponse"
        self.attribute_typecode_dict = {}
        ZSI.TCcompound.ComplexType.__init__(self, None, TClist, inorder=0, mixed=True, mixed_aname="_text", **kw)
        class Holder:
            __metaclass__ = pyclass_type
            typecode = self
            def __init__(self):
                # pyclass
                return
        Holder.__name__ = "QueryResourcePropertiesResponse_Holder"
        self.pyclass = Holder


class ReferenceParametersType_Def(ZSI.TC.ComplexType, TypeDefinition):
    """BUG This type is not defined in ws-addressing, but it's being used.
    <wsa:ReferenceParameters xsi:type="wsa:ReferenceParametersType"/>
    """
    schema = 'http://schemas.xmlsoap.org/ws/2004/03/addressing'
    type = (schema, "ReferenceParametersType")
    
    def __init__(self, pname, **kw):
        ZSI.TC.ComplexType.__init__(self, None, [], pname=pname, inorder=0, **kw)

        class Holder:
            typecode = self
            __metaclass__ = pyclass_type
            def __init__(self):
                # pyclass
                return
        Holder.__name__ = "ReferenceParametersType_Holder"
        self.pyclass = Holder


class KeyInfoType_Def(ZSI.TCcompound.ComplexType, TypeDefinition):
    """This is a patch in pyGridWare/patch.py, 
    typecode representation of choice is a limitation.


<complexType name="KeyInfoType" mixed="true">   
  <choice maxOccurs="unbounded">
    <element ref="ds:KeyName"/>
    <element ref="ds:KeyValue"/>
    <element ref="ds:RetrievalMethod"/>
    <element ref="ds:X509Data"/>
    <element ref="ds:PGPData"/>
    <element ref="ds:SPKIData"/>
    <element ref="ds:MgmtData"/>
    <any processContents="lax" namespace="##other"/>
    <!-- (1,1) elements from (0,unbounded) namespaces -->
  </choice>
  <attribute name="Id" type="ID" use="optional"/>
</complexType>

    """
    schema = "http://www.w3.org/2000/09/xmldsig#"
    type = (schema, "KeyInfoType")

    def __init__(self, pname, ofwhat=(), attributes=None, extend=False, restrict=False, **kw):
        ns = KeyInfoType_Def.schema

        TClist = map(lambda n: GED(ns,n,isref=True)(minOccurs=0,maxOccurs=1,nillable=0),
            ("KeyName", "KeyValue", "RetrievalMethod", "X509Data", "PGPData", "SPKIData", "MgmtData")
        )
        TClist.append(ZSI.TC.AnyElement(aname="_any", minOccurs=0, maxOccurs=1, nillable=False, processContents="lax"))

        self.attribute_typecode_dict = attributes or {}
        if extend: TClist += ofwhat
        if restrict: TClist = ofwhat
        else:
            # attribute handling code
            self.attribute_typecode_dict["Id"] = ZSI.TC.AnyType()

        ZSI.TCcompound.ComplexType.__init__(self, None, TClist, pname=pname, inorder=0, mixed=True, mixed_aname="_text", **kw)

        class Holder:
            __metaclass__ = pyclass_type
            typecode = self
            def __init__(self):
                # pyclass
                self._KeyName = None
                self._KeyValue = None
                self._RetrievalMethod = None
                self._X509Data = None
                self._PGPData = None
                self._SPKIData = None
                self._MgmtData = None
                return

        Holder.__name__ = "KeyInfoType_Holder"
        self.pyclass = Holder


class PGPDataType_Def(ZSI.TCcompound.ComplexType, TypeDefinition):
    """This is a patch in pyGridWare/patch.py, 
    typecode representation of choice is a limitation.

<complexType name="PGPDataType">
  <choice>
    <sequence>
      <element name="PGPKeyID" type="base64Binary"/>   
      <element name="PGPKeyPacket" type="base64Binary" minOccurs="0"/>
      <any namespace="##other" processContents="lax" minOccurs="0"
       maxOccurs="unbounded"/>
    </sequence>
    <sequence>
      <element name="PGPKeyPacket" type="base64Binary"/>
      <any namespace="##other" processContents="lax" minOccurs="0"  
       maxOccurs="unbounded"/>
    </sequence>
  </choice>
</complexType>
    """
    schema = "http://www.w3.org/2000/09/xmldsig#"
    type = (schema, "PGPDataType")

    def __init__(self, pname, ofwhat=(), attributes=None, extend=False, restrict=False, **kw):
        ns = PGPDataType_Def.schema
        TClist = [\
            ZSI.TC.Base64String(pname=(ns,"PGPKeyID"), aname="_PGPKeyID", minOccurs=0, maxOccurs=1, nillable=False),
            ZSI.TC.Base64String(pname=(ns,"PGPKeyPacket"), aname="_PGPKeyPacket", minOccurs=0, maxOccurs=1, nillable=False),
            ZSI.TC.AnyElement(aname="_any", minOccurs=0, maxOccurs="unbounded", nillable=False, processContents="lax"), 
        ]

        self.attribute_typecode_dict = attributes or {}
        if extend: TClist += ofwhat 
        if restrict: TClist = ofwhat 
        ZSI.TCcompound.ComplexType.__init__(self, None, TClist, pname=pname, inorder=0, **kw)
        class Holder:
            __metaclass__ = pyclass_type
            typecode = self
            def __init__(self):
                # pyclass
                self._PGPKeyID = None
                self._PGPKeyPacket = None
                return
        Holder.__name__ = "PGPDataType_Holder"
        self.pyclass = Holder




