###########################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
# $Id:
from twisted.python import log
from twisted.internet import defer
from pyGridWare.resource.interfaces import IResourceHome, IWSResource
from interfaces import IX509Credential


class X509SubjectDNChecker(object):
    
    credentialInterfaces = (IX509Credential,)
    
    def requestAvatarId(self, credentials):
        """return subject DN
        """
        subject =  credentials.get_subject()
        log.msg("Subject %s" %subject, debug=True)
        if not subject:
            defer.fail(error.UnauthorizedLogin("permission denied"))
        
        return defer.succeed(subject)
    
    
class GridMapChecker(object):
    """takes an X509Credential, uses the SubjectDN to perform
    a lookup on a grid-mapfile dict.  
    
    returns the userID.
    
    class attributes:
        gridmap -- gridmap dictionary of subjectDN to user mapping.
    """
    credentialInterfaces = (IX509Credential,)
    gridmap = None
        
    @classmethod
    def load(cls, gridmapfile):
        """reload gridmap dict
        """
        fd = open(gridmapfile, 'r')    
        cls.gridmap = dict(map(lambda s: s.strip('"').split('" ')[:2], fd))
    
    def requestAvatarId(self, credentials):
        """return subject DN
        """
        if self.gridmap is None:
            defer.fail(error.UnauthorizedLogin(
                           "permission denied: no gridmap-file"))
            
        subject =  credentials.get_subject()
        log.msg("Subject %s" %subject, debug=True)
        if not subject:
            defer.fail(error.UnauthorizedLogin("permission denied"))
            
        if not self.gridmap.has_key(subject):
            defer.fail(error.UnauthorizedLogin(
               'authentication failed, no gridmap entry for subjectDN "%s"' %
               subject))
            
        #TODO: should probably check if user exists?
        user = self.gridmap.get(subject)
        if not user:
            defer.fail(error.UnauthorizedLogin(
               'authentication failed, bad gridmap entry for subjectDN "%s"' %
               subject))
            
        return defer.succeed(user)


