#!/usr/bin/env python
############################################################################
# Joshua R. Boverhof, LBNL
# See LBNLCopyright for copyright notice!
###########################################################################
from ZSI.TC import _get_global_element_declaration as GED
from ZSI.wstools.Namespaces import SOAP, BEA, WSSE, GLOBUS, WSTRUST, OASIS

import pyGridWare.generated.types.secureconversation
from pyGridWare.generated.stubs.secureconversation import\
    SecureConversationServiceLocator, RequestSecurityTokenMsg,\
    RequestSecurityTokenResponseMsg

from M2Crypto import SSL    
#from pyGridWare.security.m2.M2SecureConversation import M2SecurityContext
from M2SecurityContext import GSIAdapter as _GSIAdapter
#
#TODO: Change name of this module, this is secure conversation client utility.
#

# GED Typecodes
RequestType  = GED(WSSE.TRUST, "RequestType")
Token = GED(WSSE.TRUST,"TokenType")
BinaryExchange = GED(WSSE.TRUST,"BinaryExchange")
SecurityContextToken = GED(BEA.SECCONV, 'SecurityContextToken')
ContextAddress = GED(GLOBUS.SECCONV, 'ContextAddress')

# GED pyclass
TokenPyclass = Token.pyclass
BinaryExchangePyclass = BinaryExchange.pyclass

#  
# TODO: As of ZSI-2.0RC2 Union has no pyclass
#  GED RequestType --> GTD RequestTypeOpenEnum --> 
#  union of anyURI and RequestTypeEnum
#  
class RequestTypePyclass(str): typecode = RequestType


# Extra Namespaces
MECHANISM = "http://www.globus.org/2002/04/xmlenc#gssapi-gsi"
SECURECONV_TYPES = "http://wsrf.globus.org/security/authentication/secureconv/types"


# Functions
_is_binary_exchange = \
    lambda pyobj: getattr(pyobj, 'typecode', None) is BinaryExchange
_is_secure_context_token = \
    lambda pyobj: getattr(pyobj, 'typecode', None) is SecurityContextToken
_is_context_address = \
    lambda pyobj: getattr(pyobj, 'typecode', None) is ContextAddress
    


from pyGridWare.security.m2.ResourceHome import ManagerHome


class SecureConversationClient:
    home = ManagerHome.getInstance()
    mechanism = MECHANISM
    namespaceURI = SECURECONV_TYPES 

    def getMechanism(cls):
        return cls.mechanism
    getMechanism = classmethod(getMechanism)

    def getNamespace(cls):
        return cls.namespaceURI
    getNamespace = classmethod(getNamespace)

    def _requestSecurityToken(self,port,ctx):
        """
        """
        binEx = BinaryExchangePyclass(ctx.initSecurityContext())
        binEx.set_attribute_EncodingType(OASIS.X509TOKEN.Base64Binary)
        binEx.set_attribute_ValueType(GLOBUS.TOKEN)

        # TODO: WSTRUST.ISSUE is wrong??
        #   RequestTypePyclass(WSTRUST.ISSUE)
        request = RequestSecurityTokenMsg()
        request.Any = [
             RequestTypePyclass("http://schemas.xmlsoap.org/ws/2004/04/security/trust/Issue"),
             TokenPyclass(BEA.SCTOKEN),
             binEx,]
        
        return port.RequestSecurityToken(request)
        
    def _requestSecurityTokenResponse(self, port,ctx, response):
        """
        """
        serverTokens = filter(_is_binary_exchange, response.Any)
        if len(serverTokens) != 1:
            raise RuntimeError,\
                '%s _requestSecurityTokenResponse expecting 1 BinaryExchange, received %d' %(
                self.__class__, len(serverTokens))
        
        binEx = BinaryExchangePyclass(ctx.initSecurityContext(serverTokens[0]))
        binEx.set_attribute_EncodingType(OASIS.X509TOKEN.Base64Binary)
        binEx.set_attribute_ValueType(GLOBUS.TOKEN)
        
        request = RequestSecurityTokenResponseMsg()        
        request.set_attribute_Context(response.get_attribute_Context())
        #request.Any = [binEx]
        request.Any = [
             RequestTypePyclass("http://schemas.xmlsoap.org/ws/2004/04/security/trust/Issue"),
             TokenPyclass(BEA.SCTOKEN),
             binEx,]
    
        return port.RequestSecurityTokenResponse(request)

    def exchange(self, ctx, delegation=False, **kw):
        """client-side, use secure conversation to establish a 
        security context which is stored in M2SecurityContext
        ResourceHome and available via contextID.

        Parameters:
            ctx -- SSL.Context instance

        KeyWord Parameters:
            delegation -- TODO

        """
        assert isinstance(ctx, SSL.Context), '' 
        locator = SecureConversationServiceLocator()
        port = locator.getSecureConversation(**kw)
       
        #ctx = SSL.Context('sslv3')
        #ctx.set_info_callback()        
        #from pyGsi import proxylib
        #ctx.load_cert_chain(cert)

        context = _GSIAdapter(ctx, delegation=delegation)
#        if delegation is True:
#            context.setDelegation()
            
        response = self._requestSecurityToken(port, context)

        # Continue until receive a SecureContextToken
        l = 0
        while l == 0:
            response = self._requestSecurityTokenResponse(port,context,response)
            sct = filter(_is_secure_context_token, response.Any)
            l = len(sct)
            
        if l > 1:
            raise RuntimeError, 'expecting 1 SecureContextToken got %d' %l
                
        sct = sct[0]
        
        # Assuming only 1 context address
        address = filter(_is_context_address, sct.Any)
        try:
            address = address[0]
        except IndexError:
            raise RuntimeError, 'Expecting 1 ContextAddress got %d' %(
                len(address))

        # Assuming contextID is here
        try:
            contextID = address.ReferenceProperties.Any[0]
        except IndexError:
            raise RuntimeError, 'Can not find context key'

        context.setContextID(contextID)
#        home = M2SecurityContext.getResourceHome()
        self.home.add(context, contextID)
        
        return contextID

    authenticate = exchange

# BAD NAME, SCS is not authentication
M2AuthenticationHandler = SecureConversationClient



