#!/usr/bin/env python
##########################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
import unittest, os
from ZSI import ZSIException
import counter
from os.path import abspath, join, sep
_resolvepath = lambda f: abspath(join(*f.split('/')))
WSDL = counter.WSDL


class WrappedCounterTestCase(counter.CounterTestCase):
    """Wrap a counter test case for several purposes.  
    """
    path = '/wsrf/services/PortalCounterService'
    #path = '/wsrf/services/CounterService'
    
    def __init__(self, klass, key=None, cert=None, calist=None, **kw):
        """Instance Attributes:
        key --
        cert --
        calist -- 
        """
        self.key = key
        self.cert = cert
        self.calist = calist or []
        unittest.TestCase.__init__(self, **kw)
        self.wrappedTest = klass(**kw)

    def getEPR(self):
        return self.wrappedTest.getEPR()

    def setEPR(self, epr):
        self.wrappedTest.setEPR(epr)

    def NewRequestInstance(self):
        raise NotImplementedError, 'this method is not implemented'
        
    def setUp(self):
        kw = self.client_kw
        if kw['contextFactory'].__class__.__module__ != 'pyGridWare.utility.ssl':
            raise RuntimeError, 'expecting pyOpenSSL for client'
        
        from twisted.internet.ssl import SSL
        from pyGridWare.utility.ssl import ClientContextFactory

        key = self.key
        cert = self.cert
        calist = self.calist
        # replacement contextFactory? 
        if not (key is None and cert is None):
            f = filter(lambda f: not os.path.isfile(f), [key, cert]+calist)
            if len(f): 
                raise RuntimeError, 'missing files %s' %str(f)
    
            kw['contextFactory'] =\
                ClientContextFactory(key, cert, sslmethod=SSL.SSLv3_METHOD, 
                                     calist=calist)
                
        self.wrappedTest.client_kw = kw
        self.wrappedTest.setUp()
        
    def tearDown(self):
        self.shouldStop = True

    def runTest(self):
        self.wrappedTest.runTest()

       
 
class AccessErrorTestCase(WrappedCounterTestCase):
    """Expects the test case to raise a ZSIException.
    """
    def runTest(self):
        self.failUnlessRaises(ZSIException, self.wrappedTest.runTest)
        



def BasicServiceInstanceTestSuite():
    """Create new SSL Context in order to use a different credential,
    then try to access a service resource instance created with the
    original credential and protected by ResourcePortal.

    Receives a "ResourceUnknownFault", because we can not see the
    the resource as a different user.
    """
    kw1 = {}
    kw1['key'] = _resolvepath('etc/certificates/client2.key')
    kw1['cert'] =  _resolvepath('etc/certificates/client2.pem')
    kw1['calist'] = map(_resolvepath, ['etc/certificates/085e5b5c.0',])

    return \
        counter.CounterTestSuite(\
            tests=(WrappedCounterTestCase(counter.CreateTestCase), 
                   WrappedCounterTestCase(counter.AddTestCase), 
                   AccessErrorTestCase(counter.AddTestCase, **kw1),
                   WrappedCounterTestCase(counter.DestroyTestCase),
            )
        )


def ServiceInstanceTestSuite():
    """Create new SSL Context in order to use a different credential,
    then try to access a service resource instance created with the
    original credential and protected by ResourcePortal.

    Receives a "ResourceUnknownFault", because we can not see the
    the resource as a different user.
    """
    kw1 = {}
    kw1['key'] = _resolvepath('etc/certificates/client2.key')
    kw1['cert'] =  _resolvepath('etc/certificates/client2.pem')
    kw1['calist'] = map(_resolvepath, ['etc/certificates/085e5b5c.0',])

    return \
        counter.CounterTestSuite(\
            tests=(WrappedCounterTestCase(counter.CreateTestCase), 
                   WrappedCounterTestCase(counter.AddTestCase), 
                   AccessErrorTestCase(counter.AddTestCase, **kw1),
                   WrappedCounterTestCase(counter.CreateTestCase, **kw1),
                   WrappedCounterTestCase(counter.AddTestCase, **kw1),
                   AccessErrorTestCase(counter.AddTestCase),
                   WrappedCounterTestCase(counter.DestroyTestCase, **kw1),
            )
        )
        
        
